use actix_identity::Identity;
use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use chrono::{NaiveDate, Utc};
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::{Area, Availabillity, Event, Function, User, Role};

use super::{
    delete_availabillity::delete_availabillity,
    get_availabillity_new::get_availabillity_new,
    post_availabillity::post_availabillity,
    update_availabillity::{get_update_availabillity, post_update_availabillity},
};

pub fn init(cfg: &mut web::ServiceConfig) {
    cfg.service(get_index);
    cfg.service(get_availabillity_new);
    cfg.service(post_availabillity);
    cfg.service(delete_availabillity);
    cfg.service(get_update_availabillity);
    cfg.service(post_update_availabillity);
}

#[derive(Deserialize)]
pub struct CalendarQuery {
    date: Option<NaiveDate>,
}

#[derive(Template)]
#[template(path = "index.html")]
struct CalendarTemplate {
    user: User,
    date: NaiveDate,
    area: Area,
    events: Vec<Event>,
    availabillities: Vec<Availabillity>,
}

#[actix_web::get("/")]
async fn get_index(
    user: Identity,
    pool: web::Data<PgPool>,
    query: web::Query<CalendarQuery>,
) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();
    let date = match query.date {
        Some(given_date) => given_date,
        None => Utc::now().date_naive(),
    };
    let area = Area::read_by_id(pool.get_ref(), current_user.area_id)
        .await
        .unwrap();
    let events = Event::read_by_date(pool.get_ref(), date).await.unwrap();
    let availabillities = Availabillity::read_by_date_including_user(pool.get_ref(), date)
        .await
        .unwrap();

    let template = CalendarTemplate {
        user: current_user,
        date,
        area,
        events,
        availabillities,
    };

    HttpResponse::Ok().body(template.render().unwrap())
}
