use actix_identity::Identity;
use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use chrono::{NaiveDate, NaiveTime};
use sqlx::PgPool;

use crate::{calendar::post_availabillity::AvailabillityForm, models::{Availabillity, User, Role}};

use super::delete_availabillity::AvailabillityPath;

#[derive(Template)]
#[template(path = "availabillity_edit.html")]
pub struct AvailabillityEditTemplate {
    user: User,
    date: NaiveDate,
    id: i32,
    start_time: String,
    end_time: String,
    has_time: bool,
    comment: String,
}

#[actix_web::get("/availabillity/edit/{id}")]
pub async fn get_update_availabillity(
    user: Identity,
    pool: web::Data<PgPool>,
    path: web::Path<AvailabillityPath>,
) -> impl Responder {
    let current_user = User::read_by_id(pool.as_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();

    if let Ok(availabillity) = Availabillity::read_by_id(pool.get_ref(), path.id).await {
        if availabillity.user_id == user.id().unwrap().parse::<i32>().unwrap() {
            let start_time = availabillity
                .start_time
                .unwrap_or(NaiveTime::from_hms_opt(0, 0, 0).unwrap())
                .format("%R")
                .to_string();

            let end_time = availabillity
                .end_time
                .unwrap_or(NaiveTime::from_hms_opt(23, 59, 0).unwrap())
                .format("%R")
                .to_string();

            let has_time = availabillity.start_time.is_some() && availabillity.end_time.is_some();

            let comment = availabillity.comment.unwrap_or(String::new());

            let template = AvailabillityEditTemplate {
                user: current_user,
                date: availabillity.date,
                id: path.id,
                start_time,
                end_time,
                has_time,
                comment
            };

            return template.to_response();
        }
    }

    HttpResponse::BadRequest().body("Availabillity with this id doesn't exist.")
}

#[actix_web::post("/availabillity/edit/{id}")]
pub async fn post_update_availabillity(
    user: Identity,
    pool: web::Data<PgPool>,
    path: web::Path<AvailabillityPath>,
    form: web::Form<AvailabillityForm>
) -> impl Responder {
    if let Ok(mut availabillity) = Availabillity::read_by_id(pool.get_ref(), path.id).await {
        if availabillity.user_id == user.id().unwrap().parse::<i32>().unwrap() {
            let mut has_changed = false;

            if availabillity.start_time != form.from {
                availabillity.start_time = form.from;
                has_changed = true;
            }

            if availabillity.end_time != form.till {
                availabillity.end_time = form.till;
                has_changed = true;
            }

            if availabillity.comment != form.comment {
                availabillity.comment = form.comment.clone();
                has_changed = true;
            }

            if has_changed {
                if let Ok(_) = Availabillity::update(pool.get_ref(), path.id, &availabillity).await {
                    return HttpResponse::Found()
                        .insert_header((LOCATION, "/"))
                        .finish();
                }
            }
        }
    }

    HttpResponse::BadRequest().body("Fehler beim erstellen")
}
