use actix_identity::Identity;
use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use sqlx::PgPool;

use crate::models::{Area, Role, User};

#[derive(Template)]
#[template(path = "location/new.html")]
pub struct NewLocationTemplate {
    user: User,
    areas: Option<Vec<Area>>,
}

#[actix_web::get("/locations/new")]
pub async fn get(user: Identity, pool: web::Data<PgPool>) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();

    if current_user.role == Role::AreaManager || current_user.role == Role::Admin {
        let mut areas = None;

        if current_user.role == Role::Admin {
            areas = Some(Area::read_all(pool.get_ref()).await.unwrap());
        }

        let template = NewLocationTemplate {
            user: current_user,
            areas,
        };

        return template.to_response();
    }

    return HttpResponse::Unauthorized().finish();
}
